@@ -266,7 +266,9 @@ class LoopTokenReducer:
         selected_messages = []
         token_count = 0
         for msg in messages[::-1]:
-            msg_token_count = self._count_tokens(str(msg.content))
+            msg_token_count = self._count_message_tokens(
+                LanguageModelMessages(root=[msg])
+            )
             if token_count + msg_token_count > token_limit:
                 break
             selected_messages.append(msg)
@@ -293,9 +295,6 @@ class LoopTokenReducer:
                 )
         return messages
 
-    def _count_tokens(self, text: str) -> int:
-        return len(self._encoder.encode(text))
-
     def ensure_last_message_is_user_message(self, limited_history_messages):
         """
         As the token limit can be reached in the middle of a gpt_request,
