@@ -1,5 +1,4 @@
 import logging
-from collections import defaultdict
 from typing import override
 
 import unique_sdk
@@ -12,8 +11,16 @@ from unique_toolkit.agentic.evaluation.schemas import (
     EvaluationMetricName,
     EvaluationMetricResult,
 )
-from unique_toolkit.agentic.tools.a2a.evaluation.config import SubAgentEvaluationConfig
-from unique_toolkit.agentic.tools.a2a.service import SubAgentTool
+from unique_toolkit.agentic.tools.a2a.evaluation._utils import (
+    _get_valid_assessments,
+    _sort_assessments,
+    _worst_label,
+)
+from unique_toolkit.agentic.tools.a2a.evaluation.config import (
+    SubAgentEvaluationConfig,
+    SubAgentEvaluationServiceConfig,
+)
+from unique_toolkit.agentic.tools.a2a.tool import SubAgentTool
 from unique_toolkit.chat.schemas import (
     ChatMessageAssessmentLabel,
     ChatMessageAssessmentStatus,
@@ -27,20 +34,19 @@ logger = logging.getLogger(__name__)
 
 
 class _SubAgentToolInfo(TypedDict):
-    assessments: list[list[unique_sdk.Space.Assessment]]
+    assessments: dict[int, list[unique_sdk.Space.Assessment]]
     display_name: str
 
 
 NO_ASSESSMENTS_FOUND = "NO_ASSESSMENTS_FOUND"
 
 
-class SubAgentsEvaluation(Evaluation):
+class SubAgentEvaluationService(Evaluation):
     DISPLAY_NAME = "Sub Agents"
 
     def __init__(
         self,
-        config: SubAgentEvaluationConfig,
-        sub_agent_tools: list[SubAgentTool],
+        config: SubAgentEvaluationServiceConfig,
         language_model_service: LanguageModelService,
     ):
         super().__init__(EvaluationMetricName.SUB_AGENT)
@@ -49,14 +55,6 @@ class SubAgentsEvaluation(Evaluation):
         self._assistant_id_to_tool_info: dict[str, _SubAgentToolInfo] = {}
         self._language_model_service = language_model_service
 
-        for sub_agent_tool in sub_agent_tools:
-            if sub_agent_tool.config.evaluation_config.display_evalution:
-                sub_agent_tool.subscribe(self)
-                self._assistant_id_to_tool_info[sub_agent_tool.config.assistant_id] = {
-                    "assessments": [],
-                    "display_name": sub_agent_tool.display_name(),
-                }
-
     @override
     def get_assessment_type(self) -> ChatMessageAssessmentType:
         return self._config.assessment_type
@@ -71,102 +69,46 @@ class SubAgentsEvaluation(Evaluation):
 
         value = ChatMessageAssessmentLabel.GREEN
 
-        # Use a dict in order to compare labels (RED being the worst)
-        label_comparison_dict = defaultdict(
-            lambda: 3
-        )  # Unkown labels are highest in the sorting
-        label_comparison_dict[ChatMessageAssessmentLabel.GREEN] = 2
-        label_comparison_dict[ChatMessageAssessmentLabel.YELLOW] = 1
-        label_comparison_dict[ChatMessageAssessmentLabel.RED] = 0
-
-        for assistant_id, tool_info in self._assistant_id_to_tool_info.items():
+        for tool_info in self._assistant_id_to_tool_info.values():
             sub_agent_assessments = tool_info["assessments"] or []
-            for i, assessments in enumerate(sub_agent_assessments, start=1):
-                valid_assessments = []
-                for assessment in assessments:
-                    if (
-                        assessment["label"] is None
-                        or assessment["label"] not in ChatMessageAssessmentLabel
-                    ):
-                        logger.warning(
-                            "Unkown assistant label %s for assistant %s will be ignored",
-                            assessment["label"],
-                            assistant_id,
-                        )
-                        continue
-                    if assessment["status"] != ChatMessageAssessmentStatus.DONE:
-                        logger.warning(
-                            "Assessment %s for assistant %s is not done (status: %s) will be ignored",
-                            assessment["label"],
-                            assistant_id,
-                        )
-                        continue
-                    valid_assessments.append(assessment)
+            display_name = tool_info["display_name"]
 
+            for sequence_number in sorted(sub_agent_assessments):
+                assessments = sub_agent_assessments[sequence_number]
+
+                valid_assessments = _get_valid_assessments(
+                    assessments, display_name, sequence_number
+                )
                 if len(valid_assessments) == 0:
                     logger.info(
-                        "No valid assessment found for assistant %s", assistant_id
+                        "No valid assessment found for assistant %s (sequence number: %s)",
+                        display_name,
+                        sequence_number,
                     )
                     continue
 
-                assessments = sorted(
-                    valid_assessments, key=lambda x: label_comparison_dict[x["label"]]
-                )
+                assessments = _sort_assessments(valid_assessments)
+                value = _worst_label(value, assessments[0]["label"])  # type: ignore
 
-                for assessment in assessments:
-                    value = min(
-                        value,
-                        assessment["label"],
-                        key=lambda x: label_comparison_dict[x],
-                    )
                 data = {
                     "name": tool_info["display_name"],
                     "assessments": assessments,
                 }
                 if len(sub_agent_assessments) > 1:
-                    data["name"] += f" {i}"
+                    data["name"] += f" {sequence_number}"
 
                 sub_agents_display_data.append(data)
 
         if len(sub_agents_display_data) == 0:
             logger.warning("No valid sub agent assessments found")
+
             return EvaluationMetricResult(
                 name=self.get_name(),
                 value=NO_ASSESSMENTS_FOUND,
                 reason="No sub agents assessments found",
             )
 
-        should_summarize = False
-        reason = ""
-
-        if len(sub_agents_display_data) > 1:
-            should_summarize = True
-        elif len(sub_agents_display_data) == 1:
-            if len(sub_agents_display_data[0]["assessments"]) > 1:
-                should_summarize = True
-            else:
-                reason = (
-                    sub_agents_display_data[0]["assessments"][0]["explanation"] or ""
-                )
-
-        if should_summarize:
-            messages = (
-                MessagesBuilder()
-                .system_message_append(self._config.summarization_system_message)
-                .user_message_append(
-                    Template(self._config.summarization_user_message_template).render(
-                        sub_agents=sub_agents_display_data,
-                    )
-                )
-                .build()
-            )
-
-            reason = await self._language_model_service.complete_async(
-                messages=messages,
-                model_name=self._config.summarization_model.name,
-                temperature=0.0,
-            )
-            reason = str(reason.choices[0].message.content)
+        reason = await self._get_reason(sub_agents_display_data)
 
         return EvaluationMetricResult(
             name=self.get_name(),
@@ -196,8 +138,30 @@ class SubAgentsEvaluation(Evaluation):
             type=self.get_assessment_type(),
         )
 
+    def register_sub_agent_tool(
+        self, tool: SubAgentTool, evaluation_config: SubAgentEvaluationConfig
+    ) -> None:
+        if not evaluation_config.include_evaluation:
+            logger.warning(
+                "Sub agent tool %s has evaluation config `include_evaluation` set to False, responses will be ignored.",
+                tool.config.assistant_id,
+            )
+            return
+
+        if tool.config.assistant_id not in self._assistant_id_to_tool_info:
+            tool.subscribe(self)
+            self._assistant_id_to_tool_info[tool.config.assistant_id] = (
+                _SubAgentToolInfo(
+                    display_name=tool.display_name(),
+                    assessments={},
+                )
+            )
+
     def notify_sub_agent_response(
-        self, sub_agent_assistant_id: str, response: unique_sdk.Space.Message
+        self,
+        response: unique_sdk.Space.Message,
+        sub_agent_assistant_id: str,
+        sequence_number: int,
     ) -> None:
         if sub_agent_assistant_id not in self._assistant_id_to_tool_info:
             logger.warning(
@@ -206,10 +170,39 @@ class SubAgentsEvaluation(Evaluation):
             )
             return
 
-        self._assistant_id_to_tool_info[sub_agent_assistant_id]["assessments"].append(
+        sub_agent_assessments = self._assistant_id_to_tool_info[sub_agent_assistant_id][
+            "assessments"
+        ]
+        sub_agent_assessments[sequence_number] = (
             response[
                 "assessment"
             ].copy()  # Shallow copy as we don't modify individual assessments
             if response["assessment"] is not None
             else []
         )
+
+    async def _get_reason(self, sub_agents_display_data: list[dict]) -> str:
+        if (
+            len(sub_agents_display_data) == 1
+            and len(sub_agents_display_data[0]["assessments"]) == 1
+        ):
+            return sub_agents_display_data[0]["assessments"][0]["explanation"] or ""
+
+        messages = (
+            MessagesBuilder()
+            .system_message_append(self._config.summarization_system_message)
+            .user_message_append(
+                Template(self._config.summarization_user_message_template).render(
+                    sub_agents=sub_agents_display_data,
+                )
+            )
+            .build()
+        )
+
+        reason = await self._language_model_service.complete_async(
+            messages=messages,
+            model_name=self._config.summarization_model.name,
+            temperature=0.0,
+        )
+
+        return str(reason.choices[0].message.content)
